@@ -97,16 +97,21 @@ samplv1widget::samplv1widget ( QWidget *pParent )
 
 	// Offset/Loop range font.
 	const QFont& font = m_ui.Gen1ReverseKnob->font();
-	m_ui.Gen1OffsetRangeLabel->setFont(font);
+	const QFont font2(font.family(), font.pointSize() - 1);
+	m_ui.Gen1OctavesTextLabel->setFont(font2);
+	m_ui.Gen1OctavesComboBox->setFont(font);
+	m_ui.Gen1OffsetRangeLabel->setFont(font2);
 	m_ui.Gen1OffsetStartSpinBox->setFont(font);
 	m_ui.Gen1OffsetEndSpinBox->setFont(font);
-	m_ui.Gen1LoopRangeLabel->setFont(font);
+	m_ui.Gen1LoopRangeLabel->setFont(font2);
 	m_ui.Gen1LoopStartSpinBox->setFont(font);
 	m_ui.Gen1LoopEndSpinBox->setFont(font);
 	m_ui.Gen1LoopFadeCheckBox->setFont(font);
-	m_ui.Gen1LoopFadeSpinBox->setFont(font);
+	m_ui.Gen1LoopFadeSpinBox->setFont(font2);
 
 	const QFontMetrics fm(font);
+	m_ui.Gen1OctavesComboBox->setMaximumWidth(76);
+	m_ui.Gen1OctavesComboBox->setMaximumHeight(fm.height() + 6);
 	m_ui.Gen1OffsetStartSpinBox->setMaximumHeight(fm.height() + 6);
 	m_ui.Gen1OffsetEndSpinBox->setMaximumHeight(fm.height() + 6);
 	m_ui.Gen1LoopStartSpinBox->setMaximumHeight(fm.height() + 6);
@@ -125,6 +130,13 @@ samplv1widget::samplv1widget ( QWidget *pParent )
 	m_ui.Gen1LoopEndSpinBox->setMinimum(0);
 	m_ui.Gen1LoopFadeSpinBox->setMinimum(0);
 
+	// Sample octave tables (±otabs).
+	m_ui.Gen1OctavesComboBox->clear();
+	m_ui.Gen1OctavesComboBox->addItem(tr("None"));
+	m_ui.Gen1OctavesComboBox->addItem(tr("1 octave"));
+	for (int otabs = 2; otabs <= 4; ++otabs)
+		m_ui.Gen1OctavesComboBox->addItem(tr("%1 octaves").arg(otabs));
+
 	// Note names.
 	QStringList notes;
 	for (int note = 0; note < 128; ++note)
@@ -495,6 +507,9 @@ samplv1widget::samplv1widget ( QWidget *pParent )
 	QObject::connect(m_ui.Gen1Sample,
 		SIGNAL(loadSampleFile(const QString&)),
 		SLOT(loadSample(const QString&)));
+	QObject::connect(m_ui.Gen1OctavesComboBox,
+		SIGNAL(activated(int)),
+		SLOT(octavesChanged(int)));
 
 	// Preset management
 	QObject::connect(m_ui.Preset,
@@ -906,12 +921,12 @@ void samplv1widget::randomParams (void)
 		const samplv1::ParamIndex index = samplv1::ParamIndex(i);
 		// Filter out some non-randomizable parameters!...
 		if (index == samplv1::GEN1_SAMPLE   ||
-		    index == samplv1::GEN1_OFFSET   ||
-		    index == samplv1::GEN1_OFFSET_1 ||
-		    index == samplv1::GEN1_OFFSET_2 ||
-		    index == samplv1::GEN1_LOOP     ||
-		    index == samplv1::GEN1_LOOP_1   ||
-		    index == samplv1::GEN1_LOOP_2   ||
+			index == samplv1::GEN1_OFFSET   ||
+			index == samplv1::GEN1_OFFSET_1 ||
+			index == samplv1::GEN1_OFFSET_2 ||
+			index == samplv1::GEN1_LOOP     ||
+			index == samplv1::GEN1_LOOP_1   ||
+			index == samplv1::GEN1_LOOP_2   ||
 			index == samplv1::GEN1_OCTAVE   ||
 			index == samplv1::GEN1_TUNING   ||
 			index == samplv1::GEN1_ENVTIME  ||
@@ -1042,6 +1057,8 @@ void samplv1widget::resetParamValues (void)
 // Reset all knob default values.
 void samplv1widget::resetParamKnobs (void)
 {
+	m_ui.Gen1OctavesComboBox->setCurrentIndex(0);
+
 	for (uint32_t i = 0; i < samplv1::NUM_PARAMS; ++i) {
 		samplv1widget_param *pParam = paramKnob(samplv1::ParamIndex(i));
 		if (pParam)
@@ -1131,16 +1148,24 @@ void samplv1widget::clearSample (void)
 }
 
 
-// Sample file loader slot.
+// Sample file loader slots.
 void samplv1widget::loadSample ( const QString& sFilename )
 {
 	const QFileInfo info(sFilename);
+	const int iOctaves
+		= m_ui.Gen1OctavesComboBox->currentIndex();
 
-	loadSampleFile(info.canonicalFilePath());
+	loadSampleFile(info.canonicalFilePath(), iOctaves);
+}
 
-	m_ui.StatusBar->showMessage(
-		tr("Load sample: %1").arg(info.fileName()), 5000);
-	updateDirtyPreset(true);
+
+void samplv1widget::octavesChanged ( int iOctaves )
+{
+	samplv1_sample *pSample = m_ui.Gen1Sample->sample();
+	if (pSample == nullptr)
+		return;
+
+	loadSampleFile(pSample->filename(), iOctaves);
 }
 
 
@@ -1160,24 +1185,28 @@ void samplv1widget::clearSampleFile (void)
 
 	samplv1_ui *pSamplUi = ui_instance();
 	if (pSamplUi)
-		pSamplUi->setSampleFile(nullptr);
+		pSamplUi->setSampleFile(nullptr, 0);
 
 	updateSample(nullptr);
 }
 
 
 // Sample file loader.
-void samplv1widget::loadSampleFile ( const QString& sFilename )
+void samplv1widget::loadSampleFile ( const QString& sFilename, int iOctaves )
 {
 #ifdef CONFIG_DEBUG
-	qDebug("samplv1widget::loadSampleFile(\"%s\")", sFilename.toUtf8().constData());
+	qDebug("samplv1widget::loadSampleFile(\"%s\", %d)", sFilename.toUtf8().constData(), iOctaves);
 #endif
 
 	samplv1_ui *pSamplUi = ui_instance();
 	if (pSamplUi) {
-		pSamplUi->setSampleFile(sFilename.toUtf8().constData());
+		pSamplUi->setSampleFile(sFilename.toUtf8().constData(), iOctaves);
 		updateSample(pSamplUi->sample());
 	}
+
+	m_ui.StatusBar->showMessage(
+		tr("Load sample: %1 (%2)").arg(sFilename).arg(iOctaves), 5000);
+	updateDirtyPreset(true);
 }
 
 
@@ -1385,6 +1414,7 @@ void samplv1widget::loopRangeChanged (void)
 void samplv1widget::updateOffsetLoop ( samplv1_sample *pSample, bool bDirty )
 {
 	if (pSample && pSample->filename()) {
+		const uint16_t iOctaves   = pSample->otabs();
 		const bool     bOffset    = pSample->isOffset();
 		const uint32_t iOffsetStart = pSample->offsetStart();
 		const uint32_t iOffsetEnd = pSample->offsetEnd();
@@ -1395,6 +1425,7 @@ void samplv1widget::updateOffsetLoop ( samplv1_sample *pSample, bool bDirty )
 		const bool     bLoopZero  = pSample->isLoopZeroCrossing();
 		const uint32_t nframes    = pSample->length();
 		const float    srate      = pSample->sampleRate();
+		m_ui.Gen1OctavesComboBox->setCurrentIndex(iOctaves);
 		m_ui.Gen1OffsetRangeLabel->setEnabled(bOffset);
 		m_ui.Gen1OffsetStartSpinBox->setSampleRate(srate);
 		m_ui.Gen1OffsetStartSpinBox->setEnabled(bOffset);
@@ -1454,7 +1485,7 @@ void samplv1widget::updateOffsetLoop ( samplv1_sample *pSample, bool bDirty )
 					.arg(m_ui.Gen1Sample->textFromValue(iLoopEnd)));
 			}
 			if (!sMessage.isEmpty())
-				m_ui.StatusBar->showMessage(sMessage, 3000);
+				m_ui.StatusBar->showMessage(sMessage, 5000);
 			updateDirtyPreset(true);
 		}
 	} else {
