@@ -168,7 +168,7 @@ int samplv1widget_keybd::noteHigh (void) const
 void samplv1widget_keybd::setNoteKey ( int iNoteKey )
 {
 	if (iNoteKey >= MIN_NOTE && MAX_NOTE >= iNoteKey) {
-		m_notes[iNoteKey].rect = noteRect(iNoteKey, true);
+		m_notes[iNoteKey].path = notePath(iNoteKey, true);
 		m_iNoteKey = iNoteKey;
 	} else {
 		m_iNoteKey = -1;
@@ -187,6 +187,14 @@ int samplv1widget_keybd::noteKey (void) const
 // Piano key rectangle finder.
 QRect samplv1widget_keybd::noteRect ( int iNote, bool bOn ) const
 {
+	return notePath(iNote, bOn).boundingRect().toRect();
+}
+
+
+QPainterPath samplv1widget_keybd::notePath ( int iNote, bool bOn ) const
+{
+	QPainterPath path;
+
 	const int w = QWidget::width();
 	const int h = QWidget::height();
 
@@ -197,21 +205,25 @@ QRect samplv1widget_keybd::noteRect ( int iNote, bool bOn ) const
 	if (k >= 5) ++k;
 
 	const int nk = (iNote / 12) * 7 + (k >> 1);
-	int x2 = int(wk * float(nk));
-	int w2 = int(wn + 0.5f);
-	int h2 = h;
+	const int x2 = int(wk * float(nk));
+	const int w2 = int(wn + 0.5f);
+
+	QPainterPath path1;
+	path1.addRect(x2 + int(wk - float(w2 >> 1)), 0, w2 + 1, (h << 1) / 3);
+
 	if (k & 1) {
-		x2 += int(wk - float(w2 >> 1));
-		h2  = (h << 1) / 3;
-		++w2;
+		path = path1;
 	} else if (bOn) {
-		x2 += (w2 >> 1);
-		--w2;
+		path.addRect(x2, 0, wk, h);
+		if ((k == 0 || k == 2 || k == 6 || k == 8  || k == 10) && iNote < MAX_NOTE)
+			path = path.subtracted(path1.translated(+ 0.5f, 0.0f));
+		if ((k == 2 || k == 4 || k == 8 || k == 10 || k == 12) && iNote > MIN_NOTE)
+			path = path.subtracted(path1.translated(+ 0.5f - wk, 0.0f));
 	} else {
-		w2 <<= 1;
+		path.addRect(x2, 0, (w2 << 1), h);
 	}
 
-	return QRect(x2, 0, w2, h2);
+	return path;
 }
 
 
@@ -228,9 +240,9 @@ void samplv1widget_keybd::noteOn ( int iNote )
 
 	// Now for the sounding new one...
 	note.on = true;
-	note.rect = noteRect(iNote, true);
+	note.path = notePath(iNote, true);
 
-	QWidget::update(note.rect);
+	QWidget::update(note.path.boundingRect().toRect());
 }
 
 
@@ -247,7 +259,7 @@ void samplv1widget_keybd::noteOff ( int iNote )
 	// Now for the sounding new one...
 	note.on = false;
 
-	QWidget::update(note.rect);
+	QWidget::update(note.path.boundingRect().toRect());
 }
 
 
@@ -274,7 +286,7 @@ void samplv1widget_keybd::allNotesTimeout (void)
 		Note& note = m_notes[n];
 		if (note.on) {
 			note.on = false;
-			QWidget::update(note.rect);
+			QWidget::update(note.path.boundingRect().toRect());
 			emit noteOnClicked(n, 0);
 		}
 	}
@@ -287,7 +299,7 @@ void samplv1widget_keybd::allNotesTimeout (void)
 void samplv1widget_keybd::dragNoteOn ( const QPoint& pos )
 {
 	// Compute new key cordinates...
-	const int iNote = (NUM_NOTES * pos.x() / QWidget::width());
+	const int iNote = noteAt(pos);
 
 	if (iNote < m_iNoteLow || iNote > m_iNoteHigh || iNote == m_iNoteOn)
 		return;
@@ -324,6 +336,31 @@ void samplv1widget_keybd::dragNoteOff (void)
 }
 
 
+// Piano keyboard note descriminator.
+int samplv1widget_keybd::noteAt ( const QPoint& pos ) const
+{
+	const int w = QWidget::width();
+	const int h = QWidget::height();
+
+	const int yk = (h << 1) / 3;
+
+	int iNote = (NUM_NOTES * pos.x()) / w;
+	if (pos.y() >=  yk) {
+		int k = (iNote % 12);
+		if (k >= 5) ++k;
+		if (k & 1) {
+			const int xk = ((w * iNote) + (w >> 1)) / NUM_NOTES;
+			if (pos.x() >= xk)
+				++iNote;
+			else
+				--iNote;
+		}
+	}
+
+	return iNote;
+}
+
+
 // (Re)create the complete view pixmap.
 void samplv1widget_keybd::updatePixmap (void)
 {
@@ -391,7 +428,7 @@ void samplv1widget_keybd::updatePixmap (void)
 	m_iNoteHighX = noteRect(m_iNoteHigh).right();
 
 	if (m_iNoteKey >= MIN_NOTE && MAX_NOTE >= m_iNoteKey)
-		m_notes[m_iNoteKey].rect = noteRect(m_iNoteKey, true);
+		m_notes[m_iNoteKey].path = notePath(m_iNoteKey, true);
 }
 
 
@@ -421,7 +458,7 @@ void samplv1widget_keybd::paintEvent ( QPaintEvent *pPaintEvent )
 	for (int n = 0; n < NUM_NOTES; ++n) {
 		Note& note = m_notes[n];
 		if (note.on)
-			painter.fillRect(note.rect, rgbOver);
+			painter.fillPath(note.path, rgbOver);
 	}
 
 	// Keyboard range lines...
@@ -442,7 +479,7 @@ void samplv1widget_keybd::paintEvent ( QPaintEvent *pPaintEvent )
 	if (m_iNoteKey >= MIN_NOTE && MAX_NOTE >= m_iNoteKey) {
 		rgbOver = pal.highlight().color().lighter();
 		rgbOver.setAlpha(120);
-		painter.fillRect(m_notes[m_iNoteKey].rect, rgbOver);
+		painter.fillPath(m_notes[m_iNoteKey].path, rgbOver);
 	}
 }
 
@@ -522,30 +559,24 @@ void samplv1widget_keybd::mouseMoveEvent ( QMouseEvent *pMouseEvent )
 		break;
 	case DragNoteLow:
 		if (m_bNoteRange) {
-			const int w = QWidget::width();
-			if (w > 0) {
-				const int iNoteLow = safeNoteLow((NUM_NOTES * pos.x()) / w);
-				m_iNoteLowX = noteRect(iNoteLow).left();
-				QWidget::update();
-				QToolTip::showText(
-					QCursor::pos(),
-					tr("Low: %1 (%2)")
-						.arg(noteName(iNoteLow)).arg(iNoteLow), this);
-			}
+			const int iNoteLow = safeNoteLow(noteAt(pos));
+			m_iNoteLowX = noteRect(iNoteLow).left();
+			QWidget::update();
+			QToolTip::showText(
+				QCursor::pos(),
+				tr("Low: %1 (%2)")
+					.arg(noteName(iNoteLow)).arg(iNoteLow), this);
 		}
 		break;
 	case DragNoteHigh:
 		if (m_bNoteRange) {
-			const int w = QWidget::width();
-			if (w > 0) {
-				const int iNoteHigh = safeNoteHigh((NUM_NOTES * pos.x()) / w);
-				m_iNoteHighX = noteRect(iNoteHigh).right();
-				QWidget::update();
-				QToolTip::showText(
-					QCursor::pos(),
-					tr("High: %1 (%2)")
-						.arg(noteName(iNoteHigh)).arg(iNoteHigh), this);
-			}
+			const int iNoteHigh = safeNoteHigh(noteAt(pos));
+			m_iNoteHighX = noteRect(iNoteHigh).right();
+			QWidget::update();
+			QToolTip::showText(
+				QCursor::pos(),
+				tr("High: %1 (%2)")
+					.arg(noteName(iNoteHigh)).arg(iNoteHigh), this);
 		}
 		break;
 	case DragNoteRange:
@@ -612,20 +643,14 @@ void samplv1widget_keybd::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
 	switch (m_dragState) {
 	case DragNoteLow:
 		if (m_bNoteRange) {
-			const int w = QWidget::width();
-			if (w > 0) {
-				setNoteLow((NUM_NOTES * pos.x()) / w);
-				emit noteRangeChanged();
-			}
+			setNoteLow(noteAt(pos));
+			emit noteRangeChanged();
 		}
 		break;
 	case DragNoteHigh:
 		if (m_bNoteRange) {
-			const int w = QWidget::width();
-			if (w > 0) {
-				setNoteHigh((NUM_NOTES * pos.x()) / w);
-				emit noteRangeChanged();
-			}
+			setNoteHigh(noteAt(pos));
+			emit noteRangeChanged();
 		}
 		break;
 	case DragNoteRange:
@@ -705,9 +730,9 @@ bool samplv1widget_keybd::eventFilter ( QObject *pObject, QEvent *pEvent )
 // Present a tooltip for a note.
 void samplv1widget_keybd::noteToolTip ( const QPoint& pos ) const
 {
-	const int iNote = (NUM_NOTES * pos.x() / QWidget::width());
+	const int iNote = noteAt(pos);
 
-	if (iNote < MIN_NOTE || MAX_NOTE > iNote)
+	if (iNote < MIN_NOTE || iNote > MAX_NOTE)
 		return;
 
 	QToolTip::showText(QWidget::mapToGlobal(pos),
